@@ -2,8 +2,8 @@ using System;
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
-[assembly: ApplicationVersion ("1.7.0", "1.7.0")]
-[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "x86_64", "2010-05-05 13:05:48 PDT")]
+[assembly: ApplicationVersion ("1.7.2", "1.7.2")]
+[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "x86_64", "2010-06-30 20:23:01 EST")]
 
 [assembly: AssemblyVersion ("1.7.0.0")]
 [assembly: AssemblyTitle ("Banshee")]
