@@ -64,7 +64,7 @@ namespace Banshee.Collection.Database
         private static int last_artist_id;
         private static string last_title;
         private static DatabaseAlbumInfo last_album;
-        
+
         public static void Reset ()
         {
             last_artist_id = -1;
@@ -73,11 +73,17 @@ namespace Banshee.Collection.Database
         }
 
         public static DatabaseAlbumInfo FindOrCreate (DatabaseArtistInfo artist, string title, string title_sort, bool isCompilation)
+        {
+            return FindOrCreate (artist, title, title_sort, isCompilation, null);
+        }
+
+        public static DatabaseAlbumInfo FindOrCreate (DatabaseArtistInfo artist, string title, string title_sort, bool isCompilation, string album_musicrainz_id)
         {
             DatabaseAlbumInfo album = new DatabaseAlbumInfo ();
             album.Title = title;
             album.TitleSort = title_sort;
             album.IsCompilation = isCompilation;
+            album.MusicBrainzId = album_musicrainz_id;
             return FindOrCreate (artist, album);
         }
 
@@ -89,7 +95,7 @@ namespace Banshee.Collection.Database
             }
             return db.Query (default_select_command, artist_id, title);
         }
-        
+
         public static DatabaseAlbumInfo FindOrCreate (DatabaseArtistInfo artist, DatabaseAlbumInfo album)
         {
             if (album.Title == last_title && artist.DbId == last_artist_id && last_album != null) {
@@ -110,7 +116,7 @@ namespace Banshee.Collection.Database
                         last_album.ArtistName = artist.Name;
                         save = true;
                     }
-                    
+
                     // Ditto artist sort name
                     if (last_album.ArtistNameSort != artist.NameSort) {
                         last_album.ArtistNameSort = artist.NameSort;
@@ -129,6 +135,12 @@ namespace Banshee.Collection.Database
                         save = true;
                     }
 
+                    // If the album MusicBrainzId has changed, but is not null
+                    if (last_album.MusicBrainzId != album.MusicBrainzId && !String.IsNullOrEmpty (album.MusicBrainzId)) {
+                        last_album.MusicBrainzId = album.MusicBrainzId;
+                        save = true;
+                    }
+
                     if (save) {
                         last_album.Save ();
                     }
@@ -140,7 +152,7 @@ namespace Banshee.Collection.Database
                     last_album = album;
                 }
             }
-            
+
             last_title = album.Title;
             last_artist_id = artist.DbId;
             return last_album;
@@ -156,6 +168,7 @@ namespace Banshee.Collection.Database
                 album.ArtistName = found.ArtistName;
                 album.ArtistNameSort = found.ArtistNameSort;
                 album.IsCompilation = found.IsCompilation;
+                album.MusicBrainzId = found.MusicBrainzId;
                 album.dbid = found.DbId;
                 album.ArtistId = found.ArtistId;
                 album.Save ();
@@ -184,19 +197,19 @@ namespace Banshee.Collection.Database
             get { return artist_id; }
             set { artist_id = value; }
         }
-        
+
         [DatabaseColumn("MusicBrainzID")]
         public override string MusicBrainzId {
             get { return base.MusicBrainzId; }
             set { base.MusicBrainzId = value; }
         }
-        
+
         [DatabaseColumn]
         public override DateTime ReleaseDate {
             get { return base.ReleaseDate; }
             set { base.ReleaseDate = value; }
         }
-        
+
         [DatabaseColumn]
         public override bool IsCompilation {
             get { return base.IsCompilation; }
@@ -208,13 +221,13 @@ namespace Banshee.Collection.Database
             get { return base.Title; }
             set { base.Title = value; }
         }
-        
+
         [DatabaseColumn]
         public override string TitleSort {
             get { return base.TitleSort; }
             set { base.TitleSort = value; }
         }
-        
+
         [DatabaseColumn(Select = false)]
         internal byte[] TitleSortKey {
             get { return Hyena.StringUtil.SortKey (TitleSort ?? DisplayTitle); }
@@ -241,7 +254,7 @@ namespace Banshee.Collection.Database
         internal byte[] ArtistNameSortKey {
             get { return Hyena.StringUtil.SortKey (ArtistNameSort ?? DisplayArtistName); }
         }
-        
+
         [DatabaseColumn(Select = false)]
         internal string ArtistNameLowered {
             get { return Hyena.StringUtil.SearchKey (DisplayArtistName); }
