@@ -36,6 +36,7 @@ using Gtk;
 using Hyena.Gui;
 using Hyena.Widgets;
 
+using Banshee.Base;
 using Banshee.Kernel;
 using Banshee.Sources;
 using Banshee.ServiceStack;
@@ -52,9 +53,9 @@ namespace Banshee.Gui.TrackEditor
     public class TrackEditorDialog : BansheeDialog
     {
         public delegate void EditorTrackOperationClosure (EditorTrackInfo track);
-    
+
         private VBox main_vbox;
-    
+
         private Frame header_image_frame;
         private Image header_image;
         private Label header_title_label;
@@ -62,59 +63,75 @@ namespace Banshee.Gui.TrackEditor
         private Label header_album_label;
         private Label edit_notif_label;
         private object tooltip_host;
-        
+
         private DateTime dialog_launch_datetime = DateTime.Now;
-        
+
         private Notebook notebook;
         public Notebook Notebook {
             get { return notebook; }
         }
-    
+
         private Button nav_backward_button;
         private Button nav_forward_button;
         private PulsingButton sync_all_button;
-        
+
         private EditorMode mode;
-        
+        internal EditorMode Mode {
+            get { return mode; }
+        }
+
+        private bool readonly_tabs = false;
+
         private List<ITrackEditorPage> pages = new List<ITrackEditorPage> ();
-        
+
         public event EventHandler Navigated;
-        
-        private TrackEditorDialog (TrackListModel model, EditorMode mode) : base (
+
+        private TrackEditorDialog (TrackListModel model, EditorMode mode)
+            : this (model, mode, false)
+        {
+        }
+
+        private TrackEditorDialog (TrackListModel model, EditorMode mode, bool readonlyTabs) : base (
             mode == EditorMode.Edit ? Catalog.GetString ("Track Editor") : Catalog.GetString ("Track Properties"))
         {
+            readonly_tabs = readonlyTabs;
             this.mode = mode;
-            
+
             LoadTrackModel (model);
-        
-            BorderWidth = 6;
-            
-            if (mode == EditorMode.Edit) {
+
+            if (mode == EditorMode.Edit || readonly_tabs) {
                 WidthRequest = 525;
-                AddStockButton (Stock.Cancel, ResponseType.Cancel);
-                AddStockButton (Stock.Save, ResponseType.Ok);
+                if (mode == EditorMode.Edit) {
+                    AddStockButton (Stock.Cancel, ResponseType.Cancel);
+                    AddStockButton (Stock.Save, ResponseType.Ok);
+                }
             } else {
-                AddStockButton (Stock.Close, ResponseType.Close, true);
                 SetSizeRequest (400, 500);
             }
-            
+
+            if (mode == EditorMode.View) {
+                AddStockButton (Stock.Close, ResponseType.Close, true);
+            }
+
             tooltip_host = TooltipSetter.CreateHost ();
-            
+
             AddNavigationButtons ();
-            
+
             main_vbox = new VBox ();
-            main_vbox.Spacing = 10;
-            main_vbox.BorderWidth = 6;
+            main_vbox.Spacing = 12;
+            main_vbox.BorderWidth = 0;
             main_vbox.Show ();
             VBox.PackStart (main_vbox, true, true, 0);
-            
+
             BuildHeader ();
             BuildNotebook ();
             BuildFooter ();
-            
+
+            LoadModifiers ();
+
             LoadTrackToEditor ();
         }
-        
+
 #region UI Building
 
         private void AddNavigationButtons ()
@@ -122,48 +139,59 @@ namespace Banshee.Gui.TrackEditor
             if (TrackCount <= 1) {
                 return;
             }
-            
+
             nav_backward_button = new Button (Stock.GoBack);
             nav_backward_button.UseStock = true;
             nav_backward_button.Clicked += delegate { NavigateBackward (); };
             nav_backward_button.Show ();
             TooltipSetter.Set (tooltip_host, nav_backward_button, Catalog.GetString ("Show the previous track"));
-            
+
             nav_forward_button = new Button (Stock.GoForward);
             nav_forward_button.UseStock = true;
             nav_forward_button.Clicked += delegate { NavigateForward (); };
             nav_forward_button.Show ();
             TooltipSetter.Set (tooltip_host, nav_forward_button, Catalog.GetString ("Show the next track"));
-            
+
             ActionArea.PackStart (nav_backward_button, false, false, 0);
             ActionArea.PackStart (nav_forward_button, false, false, 0);
             ActionArea.SetChildSecondary (nav_backward_button, true);
             ActionArea.SetChildSecondary (nav_forward_button, true);
         }
-        
+
         private void BuildHeader ()
         {
             Table header = new Table (3, 3, false);
             header.ColumnSpacing = 5;
-            
+
             header_image_frame = new Frame ();
-            
+
             header_image = new Image ();
             header_image.IconName = "media-optical";
             header_image.PixelSize = 64;
-            header_image_frame.Add (header_image);
-            
+
+            header_image_frame.Add (
+                CoverArtEditor.For (header_image,
+                    (x, y) => true,
+                    () => CurrentTrack,
+                    () => LoadCoverArt (CurrentTrack)
+                )
+            );
+
             header.Attach (header_image_frame, 0, 1, 0, 3,
-                AttachOptions.Shrink, AttachOptions.Expand, 0, 0);
-            
+                AttachOptions.Fill, AttachOptions.Expand, 0, 0);
+
             AddHeaderRow (header, 0, Catalog.GetString ("Title:"), out header_title_label);
             AddHeaderRow (header, 1, Catalog.GetString ("Artist:"), out header_artist_label);
             AddHeaderRow (header, 2, Catalog.GetString ("Album:"), out header_album_label);
-          
+
             header.ShowAll ();
             main_vbox.PackStart (header, false, false, 0);
         }
-        
+
+        private TrackInfo CurrentTrack {
+            get { return TrackCount == 0 ? null : GetTrack (CurrentTrackIndex); }
+        }
+
         private void AddHeaderRow (Table header, uint row, string title, out Label label)
         {
             Label title_label = new Label ();
@@ -171,23 +199,37 @@ namespace Banshee.Gui.TrackEditor
             title_label.Xalign = 0.0f;
             header.Attach (title_label, 1, 2, row, row + 1,
                 AttachOptions.Fill, AttachOptions.Expand, 0, 0);
-            
+
             label = new Label ();
             label.Xalign = 0.0f;
-            header.Attach (label, 2, 3, row, row + 1, 
+            label.Ellipsize = Pango.EllipsizeMode.End;
+            header.Attach (label, 2, 3, row, row + 1,
                 AttachOptions.Fill | AttachOptions.Expand, AttachOptions.Expand, 0, 0);
         }
-        
+
         private void BuildNotebook ()
         {
             notebook = new Notebook ();
             notebook.Show ();
-            
+
+            Gtk.Widget page_to_focus = null;
             foreach (TypeExtensionNode node in AddinManager.GetExtensionNodes ("/Banshee/Gui/TrackEditor/NotebookPage")) {
                 try {
                     ITrackEditorPage page = (ITrackEditorPage)node.CreateInstance ();
-                    if ((mode == EditorMode.Edit && (page.PageType == PageType.Edit || page.PageType == PageType.View)) ||
-                        (mode == EditorMode.View && (page.PageType == PageType.View || page.PageType == PageType.ViewOnly))) {
+                    bool show = false;
+                    if (mode == EditorMode.Edit && (page.PageType != PageType.ViewOnly)) {
+                        show = true;
+                    } else if (mode == EditorMode.View) {
+                        if (readonly_tabs) {
+                            show = page.PageType != PageType.EditOnly;
+                        } else {
+                            show = page.PageType == PageType.View || page.PageType == PageType.ViewOnly;
+                        }
+                    }
+                    if (show) {
+                        if (page is StatisticsPage && mode == EditorMode.View) {
+                            page_to_focus = (StatisticsPage)page;
+                        }
                         pages.Add (page);
                         page.Initialize (this);
                         page.Widget.Show ();
@@ -196,7 +238,7 @@ namespace Banshee.Gui.TrackEditor
                     Hyena.Log.Exception ("Failed to initialize NotebookPage extension node. Ensure it implements ITrackEditorPage.", e);
                 }
             }
-            
+
             pages.Sort (delegate (ITrackEditorPage a, ITrackEditorPage b) { return a.Order.CompareTo (b.Order); });
             foreach (ITrackEditorPage page in pages) {
                 Container container = page.Widget as Container;
@@ -208,19 +250,22 @@ namespace Banshee.Gui.TrackEditor
                 container.BorderWidth = 12;
                 notebook.AppendPage (container, page.TabWidget == null ? new Label (page.Title) : page.TabWidget);
             }
-            
+
             main_vbox.PackStart (notebook, true, true, 0);
+            if (page_to_focus != null) {
+                notebook.CurrentPage = notebook.PageNum (page_to_focus);
+            }
         }
-        
+
         private void BuildFooter ()
         {
-            if (mode == EditorMode.View) {
+            if (mode == EditorMode.View || TrackCount < 2) {
                 return;
             }
-        
+
             HBox button_box = new HBox ();
             button_box.Spacing = 6;
-                       
+
             if (TrackCount > 1) {
                 sync_all_button = new PulsingButton ();
                 sync_all_button.FocusInEvent += delegate {
@@ -228,22 +273,22 @@ namespace Banshee.Gui.TrackEditor
                         button.StartPulsing ();
                     });
                 };
-                
+
                 sync_all_button.FocusOutEvent += delegate {
                     if (sync_all_button.State == StateType.Prelight) {
                         return;
                     }
-                    
+
                     ForeachWidget<SyncButton> (delegate (SyncButton button) {
                         button.StopPulsing ();
                     });
                 };
-                
+
                 sync_all_button.StateChanged += delegate {
                     if (sync_all_button.HasFocus) {
                         return;
                     }
-                
+
                     ForeachWidget<SyncButton> (delegate (SyncButton button) {
                         if (sync_all_button.State == StateType.Prelight) {
                             button.StartPulsing ();
@@ -252,96 +297,92 @@ namespace Banshee.Gui.TrackEditor
                         }
                     });
                 };
-                
+
                 sync_all_button.Clicked += delegate {
                     InvokeFieldSync ();
                 };
-                
+
                 Alignment alignment = new Alignment (0.5f, 0.5f, 0.0f, 0.0f);
                 HBox box = new HBox ();
                 box.Spacing = 2;
                 box.PackStart (new Image (Stock.Copy, IconSize.Button), false, false, 0);
-                box.PackStart (new Label (Catalog.GetString ("Sync all field values")), false, false, 0);
+                box.PackStart (new Label (Catalog.GetString ("Sync all field _values")), false, false, 0);
                 alignment.Add (box);
                 sync_all_button.Add (alignment);
-                
+
                 TooltipSetter.Set (tooltip_host, sync_all_button, Catalog.GetString (
                     "Apply the values of all common fields set for this track to all of the tracks selected in this editor"));
-                    
+
                 button_box.PackStart (sync_all_button, false, false, 0);
-                
+
                 foreach (Widget child in ActionArea.Children) {
                     child.SizeAllocated += OnActionAreaChildSizeAllocated;
                 }
-                
+
                 edit_notif_label = new Label ();
                 edit_notif_label.Xalign = 1.0f;
                 button_box.PackEnd (edit_notif_label, false, false, 0);
             }
-            
+
             main_vbox.PackStart (button_box, false, false, 0);
             button_box.ShowAll ();
         }
-        
-        private delegate void WidgetAction<T> (T widget) where T : class;
-        
-        private void ForeachWidget<T> (WidgetAction<T> action) where T : class
+
+        private void LoadModifiers ()
+        {
+            foreach (TypeExtensionNode node in AddinManager.GetExtensionNodes ("/Banshee/Gui/TrackEditor/Modifier")) {
+                try {
+                    ITrackEditorModifier mod = (ITrackEditorModifier)node.CreateInstance ();
+                    mod.Modify (this);
+                } catch (Exception e) {
+                    Hyena.Log.Exception ("Failed to initialize TrackEditor/Modifier extension node. Ensure it implements ITrackEditorModifier.", e);
+                }
+            }
+        }
+
+        public void ForeachWidget<T> (WidgetAction<T> action) where T : class
         {
             for (int i = 0; i < notebook.NPages; i++) {
-                ForeachWidget (notebook.GetNthPage (i) as Container, action);
-            }     
-        }
-        
-        private void ForeachWidget<T> (Container container, WidgetAction<T> action) where T : class
-        {
-            if (container == null) {
-                return;
+                GtkUtilities.ForeachWidget (notebook.GetNthPage (i) as Container, action);
             }
-            
-            foreach (Widget child in container.Children) {
-                T widget = child as T;
-                if (widget != null) {
-                    action (widget);
-                } else {
-                    Container child_container = child as Container;
-                    if (child_container != null) {
-                        ForeachWidget<T> (child_container, action);
+        }
+
+        private void InvokeFieldSync ()
+        {
+            for (int i = 0; i < notebook.NPages; i++) {
+                var field_page = notebook.GetNthPage (i) as FieldPage;
+                if (field_page != null) {
+                    foreach (var slot in field_page.FieldSlots) {
+                        if (slot.Sync != null && (slot.SyncButton == null || slot.SyncButton.Sensitive)) {
+                            slot.Sync ();
+                        }
                     }
                 }
             }
         }
-        
-        private void InvokeFieldSync ()
-        {
-            ForeachWidget<SyncButton> (delegate (SyncButton button) {
-                if (button.Sensitive) {
-                    button.Click ();
-                }
-            });
-        }
-        
+
         private int action_area_children_allocated = 0;
-        
+
         private void OnActionAreaChildSizeAllocated (object o, SizeAllocatedArgs args)
         {
             Widget [] children = ActionArea.Children;
             if (++action_area_children_allocated != children.Length) {
                 return;
             }
-            
-            sync_all_button.WidthRequest = (children[1].Allocation.X + 
-                children[1].Allocation.Width) - children[0].Allocation.X - 1;
+
+            sync_all_button.WidthRequest = Math.Max (sync_all_button.Allocation.Width,
+                (children[1].Allocation.X + children[1].Allocation.Width) - children[0].Allocation.X - 1);
         }
-        
+
 #endregion
-        
+
 #region Track Model/Changes API
-        
+
         private CachedList<DatabaseTrackInfo> db_selection;
         private List<TrackInfo> memory_selection;
         private Dictionary<TrackInfo, EditorTrackInfo> edit_map = new Dictionary<TrackInfo, EditorTrackInfo> ();
         private int current_track_index;
-        
+
         protected void LoadTrackModel (TrackListModel model)
         {
             DatabaseTrackListModel db_model = model as DatabaseTrackListModel;
@@ -354,7 +395,7 @@ namespace Banshee.Gui.TrackEditor
                 }
             }
         }
-        
+
         public void LoadTrackToEditor ()
         {
             TrackInfo current_track = null;
@@ -362,20 +403,64 @@ namespace Banshee.Gui.TrackEditor
             if (editor_track == null) {
                 return;
             }
-            
+
             // Update the Header
             header_title_label.Text = current_track.DisplayTrackTitle;
             header_artist_label.Text = current_track.DisplayArtistName;
             header_album_label.Text = current_track.DisplayAlbumTitle;
-            
+
             if (edit_notif_label != null) {
-                edit_notif_label.Markup = String.Format (Catalog.GetString ("<i>Editing {0} of {1} items</i>"), 
+                edit_notif_label.Markup = String.Format (Catalog.GetString ("<i>Editing {0} of {1} items</i>"),
                     CurrentTrackIndex + 1, TrackCount);
             }
-            
-            ArtworkManager artwork = ServiceManager.Get<ArtworkManager> ();
-            Gdk.Pixbuf cover_art = artwork.LookupScalePixbuf (current_track.ArtworkId, 64);
+
+            LoadCoverArt (current_track);
+
+            // Disconnect all the undo adapters
+            ForeachWidget<ICanUndo> (delegate (ICanUndo undoable) {
+                undoable.DisconnectUndo ();
+            });
+
+            foreach (ITrackEditorPage page in pages) {
+                page.LoadTrack (editor_track);
+            }
+
+            // Connect all the undo adapters
+            ForeachWidget<ICanUndo> (delegate (ICanUndo undoable) {
+                undoable.ConnectUndo (editor_track);
+            });
+
+            // Update Navigation
+            if (TrackCount > 0 && nav_backward_button != null && nav_forward_button != null) {
+                nav_backward_button.Sensitive = CanGoBackward;
+                nav_forward_button.Sensitive = CanGoForward;
+            }
+
+            // If there was a widget focused already (eg the Title entry), GrabFocus on it,
+            // which causes its text to be selected, ready for editing.
+            Widget child = FocusChild;
+            while (child != null) {
+                Container container = child as Container;
+                if (container != null) {
+                    child = container.FocusChild;
+                } else if (child != null) {
+                    child.GrabFocus ();
+                    child = null;
+                }
+            }
+        }
+
+        private void LoadCoverArt (TrackInfo current_track)
+        {
+            if (current_track == null)
+                return;
+
+            var artwork = ServiceManager.Get<ArtworkManager> ();
+            var cover_art = artwork.LookupScalePixbuf (current_track.ArtworkId, 64);
+
+            header_image.Clear ();
             header_image.Pixbuf = cover_art;
+
             if (cover_art == null) {
                 header_image.IconName = "media-optical";
                 header_image.PixelSize = 64;
@@ -383,83 +468,65 @@ namespace Banshee.Gui.TrackEditor
             } else {
                 header_image_frame.ShadowType = ShadowType.In;
             }
-            
-            // Disconnect all the undo adapters
-            ForeachWidget<ICanUndo> (delegate (ICanUndo undoable) {
-                undoable.DisconnectUndo ();
-            });
-            
-            foreach (ITrackEditorPage page in pages) {
-                page.LoadTrack (editor_track);
-            }
-            
-            // Connect all the undo adapters
-            ForeachWidget<ICanUndo> (delegate (ICanUndo undoable) {
-                undoable.ConnectUndo (editor_track);
-            });
-            
-            // Update Navigation
-            if (TrackCount > 0 && nav_backward_button != null && nav_forward_button != null) {
-                nav_backward_button.Sensitive = CanGoBackward;
-                nav_forward_button.Sensitive = CanGoForward;
-            }
+
+            header_image.QueueDraw ();
         }
-        
+
         public void ForeachNonCurrentTrack (EditorTrackOperationClosure closure)
         {
             for (int i = 0; i < TrackCount; i++) {
                 if (i == current_track_index) {
                     continue;
                 }
-                
+
                 EditorTrackInfo track = LoadTrack (i);
                 if (track != null) {
                     closure (track);
                 }
             }
         }
-        
+
         public EditorTrackInfo LoadTrack (int index)
         {
             return LoadTrack (index, true);
         }
-        
+
         public EditorTrackInfo LoadTrack (int index, bool alwaysLoad)
         {
             TrackInfo source_track;
             return LoadTrack (index, alwaysLoad, out source_track);
         }
-        
+
         private EditorTrackInfo LoadTrack (int index, out TrackInfo sourceTrack)
         {
             return LoadTrack (index, true, out sourceTrack);
         }
-        
+
         private EditorTrackInfo LoadTrack (int index, bool alwaysLoad, out TrackInfo sourceTrack)
         {
             sourceTrack = GetTrack (index);
             EditorTrackInfo editor_track = null;
-            
+
             if (sourceTrack == null) {
                 // Something bad happened here
                 return null;
             }
-            
+
             if (!edit_map.TryGetValue (sourceTrack, out editor_track) && alwaysLoad) {
                 editor_track = new EditorTrackInfo (sourceTrack);
                 editor_track.EditorIndex = index;
                 editor_track.EditorCount = TrackCount;
                 edit_map.Add (sourceTrack, editor_track);
             }
-            
+
             return editor_track;
         }
-        
+
         private TrackInfo GetTrack (int index)
         {
             return db_selection != null ? db_selection[index] : memory_selection[index];
         }
-        
+
         protected virtual void OnNavigated ()
         {
             EventHandler handler = Navigated;
@@ -467,7 +534,7 @@ namespace Banshee.Gui.TrackEditor
                 handler (this, EventArgs.Empty);
             }
         }
-        
+
         public void NavigateForward ()
         {
             if (current_track_index < TrackCount - 1) {
@@ -476,7 +543,7 @@ namespace Banshee.Gui.TrackEditor
                 OnNavigated ();
             }
         }
-        
+
         public void NavigateBackward ()
         {
             if (current_track_index > 0) {
@@ -485,19 +552,19 @@ namespace Banshee.Gui.TrackEditor
                 OnNavigated ();
             }
         }
-        
+
         public int TrackCount {
             get { return db_selection != null ? db_selection.Count : memory_selection.Count; }
         }
-        
+
         public int CurrentTrackIndex {
             get { return current_track_index; }
         }
-        
+
         public bool CanGoBackward {
             get { return current_track_index > 0; }
         }
-        
+
         public bool CanGoForward {
             get { return current_track_index >= 0 && current_track_index < TrackCount - 1; }
         }
@@ -509,7 +576,7 @@ namespace Banshee.Gui.TrackEditor
         public void Save ()
         {
             List<int> primary_sources = new List<int> ();
-            
+
             // TODO: wrap in db transaction
             try {
                 DatabaseTrackInfo.NotifySaved = false;
@@ -520,9 +587,9 @@ namespace Banshee.Gui.TrackEditor
                     if (track == null || track.SourceTrack == null) {
                         continue;
                     }
-                    
+
                     SaveTrack (track);
-                    
+
                     if (track.SourceTrack is DatabaseTrackInfo) {
                         // If the source track is from the database, save its parent for notification later
                         int id = (track.SourceTrack as DatabaseTrackInfo).PrimarySourceId;
@@ -543,34 +610,17 @@ namespace Banshee.Gui.TrackEditor
                 DatabaseTrackInfo.NotifySaved = true;
             }
         }
-        
+
         private void SaveTrack (EditorTrackInfo track)
         {
             TrackInfo.ExportableMerge (track, track.SourceTrack);
             track.SourceTrack.Save ();
-                
-            if (LibrarySchema.WriteMetadata.Get ()) {
-                SaveToFile (track.SourceTrack);
-            }
 
-            if (LibrarySchema.MoveOnInfoSave.Get ()) {
-                MoveSavedFile (track.SourceTrack);
-            }
-
-            if (track.SourceTrack == ServiceManager.PlayerEngine.CurrentTrack) {
+            if (track.SourceTrack.TrackEqual (ServiceManager.PlayerEngine.CurrentTrack)) {
+                TrackInfo.ExportableMerge (track, ServiceManager.PlayerEngine.CurrentTrack);
                 ServiceManager.PlayerEngine.TrackInfoUpdated ();
             }
         }
-        
-        private void SaveToFile (TrackInfo track)
-        {
-            Scheduler.Schedule (new Banshee.Streaming.SaveTrackMetadataJob (track), JobPriority.Highest);
-        }
-
-        private void MoveSavedFile (TrackInfo track)
-        {
-            Scheduler.Schedule (new MoveOnInfoSaveJob (track), JobPriority.Highest);
-        }
 
 #endregion
 
@@ -580,15 +630,24 @@ namespace Banshee.Gui.TrackEditor
         {
             Run (model, EditorMode.Edit);
         }
-        
-        public static void RunView (TrackListModel model)
+
+        public static void RunView (TrackListModel model, bool readonlyTabs)
         {
-            Run (model, EditorMode.View);
+            Run (model, EditorMode.View, readonlyTabs);
         }
-        
+
         public static void Run (TrackListModel model, EditorMode mode)
         {
-            TrackEditorDialog track_editor = new TrackEditorDialog (model, mode);
+            Run (new TrackEditorDialog (model, mode));
+        }
+
+        private static void Run (TrackListModel model, EditorMode mode, bool readonlyTabs)
+        {
+            Run (new TrackEditorDialog (model, mode, readonlyTabs));
+        }
+
+        private static void Run (TrackEditorDialog track_editor)
+        {
             track_editor.Response += delegate (object o, ResponseArgs args) {
                 if (args.ResponseId == ResponseType.Ok) {
                     track_editor.Save ();
@@ -603,23 +662,23 @@ namespace Banshee.Gui.TrackEditor
                             }
                         }
                     }
-                    
+
                     if (changed_count == 0) {
                         track_editor.Destroy ();
                         return;
                     }
-                
+
                     HigMessageDialog message_dialog = new HigMessageDialog (
-                        track_editor, DialogFlags.Modal, MessageType.Warning, ButtonsType.None, 
-                        
+                        track_editor, DialogFlags.Modal, MessageType.Warning, ButtonsType.None,
+
                         String.Format (Catalog.GetPluralString (
                             "Save the changes made to the open track?",
                             "Save the changes made to {0} of {1} open tracks?",
                             track_editor.TrackCount), changed_count, track_editor.TrackCount),
-                            
+
                         String.Empty
                     );
-                    
+
                     UpdateCancelMessage (track_editor, message_dialog);
                     uint timeout = 0;
                     timeout = GLib.Timeout.Add (1000, delegate {
@@ -629,11 +688,11 @@ namespace Banshee.Gui.TrackEditor
                         }
                         return result;
                     });
-                    
+
                     message_dialog.AddButton (Catalog.GetString ("Close _without Saving"), ResponseType.Close, false);
                     message_dialog.AddButton (Stock.Cancel, ResponseType.Cancel, false);
                     message_dialog.AddButton (Stock.Save, ResponseType.Ok, true);
-                    
+
                     try {
                         switch ((ResponseType)message_dialog.Run ()) {
                             case ResponseType.Ok:
@@ -652,29 +711,30 @@ namespace Banshee.Gui.TrackEditor
                         message_dialog.Destroy ();
                     }
                 }
-                
+
                 track_editor.Destroy ();
             };
-            
-            track_editor.Run ();
+
+            //track_editor.Run ();
+            track_editor.Show ();
         }
-        
+
         private static bool UpdateCancelMessage (TrackEditorDialog trackEditor, HigMessageDialog messageDialog)
         {
             if (messageDialog == null) {
                 return false;
             }
-            
+
             messageDialog.MessageLabel.Text = String.Format (Catalog.GetString (
                 "If you don't save, changes from the last {0} will be permanently lost."),
                 Banshee.Sources.DurationStatusFormatters.ApproximateVerboseFormatter (
                     DateTime.Now - trackEditor.dialog_launch_datetime
                 )
             );
-            
+
             return messageDialog.IsMapped;
         }
-    
+
 #endregion
 
     }
